/*
 *  aiousb.h
 *  aiousb
 *
 *  Created by Jay Dolan on 3/14/06.
 *  This source code may be reproduced, in whole or in part. 
 *  It may be modified in any way. The only request made by ACCES is that any product
 *  using this code must interact with ACCES hardware.
 *
 */
 
#include <unistd.h>

#include <CoreFoundation/CoreFoundation.h>

#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>
#include <mach/mach.h>

#define ACCES_VENDOR_ID		0x1605

//ACCES PRODUCT IDs

#define USB_DIO_32_ID_DEV		0x8001
#define USB_DA12_8A_REVA_DEV	0xC001
#define USB_IIRO_16_DEV			0x8010
#define USB_II_16_DEV			0x8011
#define USB_RO_16_DEV			0x8012
#define USB_IIRO_8_DEV			0x8014
#define USB_II_8_DEV			0x8015
#define USB_RO_8_DEV			0x8016
#define USB_CTR_15_DEV			0x8020
#define USB_IIRO4_2SM_DEV		0x8030

//potential error codes
#define COULD_NOT_CREATE_MASTER_PORT -1
#define COULD_NOT_CREATE_DICTIONARY  -2
#define INVALID_DEVICE_INDEX		 -3
#define DEVICE_NOT_CONNECTED		 -4
#define OPERATION_NOT_SUPPORTED		 -5
#define INVALID_BYTE_INDEX			 -6
#define INVALID_BLOCK_INDEX			 -7
#define INVALID_COUNTER_INDEX		 -8
#define INVALID_BIT_INDEX			 -9
#define INVALID_ADDRESS				-10

//this will check for connected ACCES usb devices. The return value is the number of cards found. The maximum device Index for
//other functions will be the return value - 1. Returns a value 0 or greater on success, otherwise returns an error code.
//this function must be called before any others in the library, and the behavior for calling it a second time
//in the same session is undefined

SInt8 queryDevices(void);

//this will return information about the device at the
//deviceIndex: The index of the device information is being requested for
//productID: Will be set to the product ID of the device
//nameSize: the number of characters available to name, including the null terminator
//name: will be set to the name of the device
//DIOBytes: the number of DIO bytes available on the device
//Counters: the number of counter blocks available on the device 
SInt8 queryDeviceInfo(SInt8 deviceIndex, UInt16 *productID, UInt16 nameSize, char *name, UInt8 *DIOBytes, UInt8 *counters);

//configure DIO bis. if tristate = true then the bits will be disabled, otherwise they will be enabled when configuration is complete.
SInt8 dioConfig(SInt8 deviceIndex, bool tristate, UInt8 outMask, UInt8 *pBytes);


//DIO_WriteAll
//deviceIndex: The index of the device that is being written to.
//pData: pointer to an array of bytes. Must be one for each DIO byte on the board
SInt8 DIO_WriteAll (SInt8 deviceIndex, UInt8 *pData);

//DIO_Write8
//deviceIndex: The index of the device that is being written to
//byteIndex: the index of the byte to be written to.
//data: pointer to the byte to be written
SInt8 DIO_Write8 (SInt8 deviceIndex, UInt8 byteIndex, UInt8 *data);

//DIO_Write1
//deviceIndex: The index of the device that is being written to
//bitIndex: the index of the bit to be written to
//data: pointer to a boolean value where false = 0 and true = 1
SInt8 DIO_Write1 (SInt8 deviceIndex, UInt8 bitIndex, bool *data);


//DIO_ReadAll
//deviceIndex: The index of the device that is being read from
//pData: pointer to an array of bytes. Must be one for each DIO byte on the board
SInt8 DIO_ReadAll (SInt8 deviceIndex, UInt8 *pData);

//DIO_Read8
//deviceIndex: The index of the device that is being read from
//byteIndex: the index of the byte to be read from
//data: a pointer to a byte that will receive the data
SInt8 DIO_Read8 (SInt8 deviceIndex, UInt8 byteIndex, UInt8 *data);

//DIO_Read1
//deviceIndex: The index of the device that is being read from
//bitIndex: the index of the bit to be read
//data: pointer to a boolean value that will receive the value. false = 0 and true = 1
SInt8 DIO_Read1 (SInt8 deviceIndex, UInt8 bitIndex, bool *data);


//CTR_8254Mode: sets the mode of a counter on the board
//deviceIndex: The index of the device that is being written to
//blockIndex: The index of the 8254 chip that is being written to
//counterIndex: The index of the counter on the 8254 chip. (valid values are 0, 1, and 2)
//mode: The mode the counter is being set to (valid values are 0 to 5 inclusive)
SInt8 CTR_8254Mode (SInt8 deviceIndex, UInt8 blockIndex, UInt8 counterIndex, UInt8 mode);


//CTR_8254ModeLoad: sets the mode of a counter on the board
//deviceIndex: The index of the device that is being written to
//blockIndex: The index of the 8254 chip that is being written to
//counterIndex: The index of the counter on the 8254 chip. (valid values are 0, 1, and 2)
//mode: The mode the counter is being set to (valid values are 0 to 5 inclusive)
//value: The value to load into the counter (valid values are 0 to 65535 inclusive)
SInt8 CTR_8254ModeLoad (SInt8 deviceIndex, UInt8 blockIndex, UInt8 counterIndex, UInt8 mode, UInt16 value);


//CTR_8254Read: Reads the current value of a counter on the board
//deviceIndex: The index of the device that is being read from
//blockIndex: The index of the 8254 chip that is being read from
//counterIndex: The index of the counter on the 8254 chip (valid values are 0, 1, 2)
//value: pointer to a word that will receive the value
SInt8 CTR_8254Read (SInt8 deviceIndex, UInt8 blockIndex, UInt8 counterIndex, UInt16 *value);


//CTR_8254ReadAll: Reads the current values of all the counters on the board
//deviceIndex: The index of the device that is being read from
//data: a pointer to at least 2 * the number of counters on the board bytes
SInt8 CTR_8254ReadAll (SInt8 deviceIndex, UInt16 *data);

//EEPromRead: Reads data from the user accessible portion of the EEPROM
//deviceIndex: The index of the device
//address: the address to read from. Valid addresses for the user are from 0x1E00 to 0x2000 inclusive
//data: pointer to a buffer to hold the data read
//length: The length in bytes of the data to be read 
SInt8 EEPromRead (SInt8 deviceIndex, UInt16 address, UInt8 *data, SInt8 length);

//EEPromWrite: Writes data to the user accessible portion of the EEPROM
//deviceIndex: The index of the device
//address: The address to write to. Valid addresses for the user are from 0x1E00 to 0x2000 inclusive
//data: pointer to data to be written. 
//length: The length in bytes of the data to written
SInt8 EEPromWrite (SInt8 deviceIndex, UInt16 address, UInt8 *data, SInt8 length);

//DacWriteOne: Writes a value to a DAC
//deviceIndex: The index of the device
//data: The count to be written to the DAC
//channel: The channel to write the count to
SInt8 DacWriteOne (SInt8 deviceIndex, UInt16 data, SInt8 channel);

//DacWriteAll: Writes values to multiple DACs
//deviceIndex: The index of the device
//data: For more information on the format of the data for this function please consult the 
//		Vendor Requests.pdf located on the CD. The information is under the heading:
//		Vendor Request B3H: DAC Write (MULTI-CHANNEL FORM)
SInt8 DacWriteAll (SInt8 deviceIndex, UInt8 *data);

//CTR_8254ReadAllLatchedCounts
//deviceIndex: The index of the device
//data: pointer to a 17 byte buffer to recieve the counts. For more information on this buffers
//		format please see the Vendor Requests.pdf on the CD. The information is under the heading:
//		Vendor Request 26h: Read All Latched Counts (for frequency measurement)
SInt8 CTR_8254ReadAllLatchedCounts (SInt8 deviceIndex, UInt8 *data);

//CTR_8254SelectGateSource
//deviceIndex: The index of the device
//counter: The counter that is to be the gate source
SInt8 CTR_8254SelectGateSource(SInt8 deviceIndex, int counter);

